@@ -1,19 +1,22 @@
 package Email::Sender::Transport::SMTP;
 # ABSTRACT: send email over SMTP
-$Email::Sender::Transport::SMTP::VERSION = '1.300016';
+$Email::Sender::Transport::SMTP::VERSION = '1.300031';
 use Moo;
 
 use Email::Sender::Failure::Multi;
 use Email::Sender::Success::Partial;
 use Email::Sender::Role::HasMessage ();
 use Email::Sender::Util;
-use MooX::Types::MooseLike::Base qw(Bool Int Str);
+use MooX::Types::MooseLike::Base qw(Bool Int Str HashRef);
+use Net::SMTP 3.07; # SSL support, fixed datasend
+
+use utf8 (); # See below. -- rjbs, 2015-05-14
 
 #pod =head1 DESCRIPTION
 #pod
 #pod This transport is used to send email over SMTP, either with or without secure
-#pod sockets (SSL).  It is one of the most complex transports available, capable of
-#pod partial success.
+#pod sockets (SSL/TLS).  It is one of the most complex transports available, capable
+#pod of partial success.
 #pod
 #pod For a potentially more efficient version of this transport, see
 #pod L<Email::Sender::Transport::SMTP::Persistent>.
@@ -26,21 +29,57 @@ use MooX::Types::MooseLike::Base qw(Bool Int Str);
 #pod
 #pod =item C<host>: the name of the host to connect to; defaults to C<localhost>
 #pod
-#pod =item C<ssl>: if true, connect via SSL; defaults to false
+#pod =item C<ssl>: if 'starttls', use STARTTLS; if 'ssl' (or 1), connect securely;
+#pod otherwise, no security
 #pod
-#pod =item C<port>: port to connect to; defaults to 25 for non-SSL, 465 for SSL
+#pod =item C<ssl_options>: passed to Net::SMTP constructor for 'ssl' connections or
+#pod to starttls for 'starttls' connections; should contain extra options for
+#pod IO::Socket::SSL
+#pod
+#pod =item C<port>: port to connect to; defaults to 25 for non-SSL, 465 for 'ssl',
+#pod 587 for 'starttls'
 #pod
 #pod =item C<timeout>: maximum time in secs to wait for server; default is 120
 #pod
 #pod =cut
 
-has host => (is => 'ro', isa => Str,  default => sub { 'localhost' });
-has ssl  => (is => 'ro', isa => Bool, default => sub { 0 });
+sub BUILD {
+  my ($self) = @_;
+  Carp::croak("do not pass port number to SMTP transport in host, use port parameter")
+    if $self->host =~ /:/;
+}
+
+has host => (is => 'ro', isa => Str, default => sub { 'localhost' });
+has ssl  => (is => 'ro', isa => Str, default => sub { 0 });
+
+has _security => (
+  is   => 'ro',
+  lazy => 1,
+  init_arg => undef,
+  default  => sub {
+    my $ssl = $_[0]->ssl;
+    return '' unless $ssl;
+    $ssl = lc $ssl;
+    return 'starttls' if 'starttls' eq $ssl;
+    return 'ssl' if $ssl eq 1 or $ssl eq 'ssl';
+
+    Carp::cluck(qq{true "ssl" argument to Email::Sender::Transport::SMTP should be 'ssl' or 'startls' or '1' but got '$ssl'});
+
+    return 1;
+  },
+);
+
+has ssl_options => (is => 'ro', isa => HashRef, default => sub {  {}  });
+
 has port => (
   is  => 'ro',
   isa => Int,
   lazy    => 1,
-  default => sub { return $_[0]->ssl ? 465 : 25; },
+  default => sub {
+    return $_[0]->_security eq 'starttls' ? 587
+         : $_[0]->_security eq 'ssl'      ? 465
+         :                                   25
+  },
 );
 
 has timeout => (is => 'ro', isa => Int, default => sub { 120 });
@@ -87,8 +126,10 @@ sub _quoteaddr {
   my $domain     = pop @localparts;
   my $localpart  = join q{@}, @localparts;
 
-  # this is probably a little too paranoid
-  return $addr unless $localpart =~ /[^\w.+-]/ or $localpart =~ /^\./;
+  return $addr # The first regex here is RFC 821 "specials" excepting dot.
+    unless $localpart =~ /[\x00-\x1F\x7F<>\(\)\[\]\\,;:@"]/
+    or     $localpart =~ /^\./
+    or     $localpart =~ /\.$/;
   return join q{@}, qq("$localpart"), $domain;
 }
 
@@ -96,16 +137,21 @@ sub _smtp_client {
   my ($self) = @_;
 
   my $class = "Net::SMTP";
-  if ($self->ssl) {
-    require Net::SMTP::SSL;
-    $class = "Net::SMTP::SSL";
-  } else {
-    require Net::SMTP;
-  }
 
   my $smtp = $class->new( $self->_net_smtp_args );
 
-  $self->_throw("unable to establish SMTP connection") unless $smtp;
+  unless ($smtp) {
+    $self->_throw(
+      sprintf "unable to establish SMTP connection to %s port %s",
+        $self->host,
+        $self->port,
+    );
+  }
+
+  if ($self->_security eq 'starttls') {
+    $self->_throw("can't STARTTLS: " . $smtp->message)
+      unless $smtp->starttls(%{ $self->ssl_options });
+  }
 
   if ($self->sasl_username) {
     $self->_throw("sasl_username but no sasl_password")
@@ -131,6 +177,11 @@ sub _net_smtp_args {
     Port    => $self->port,
     Timeout => $self->timeout,
     Debug   => $self->debug,
+
+    (($self->_security eq 'ssl')
+      ? (SSL => 1, %{ $self->ssl_options })
+      : ()),
+
     defined $self->helo      ? (Hello     => $self->helo)      : (),
     defined $self->localaddr ? (LocalAddr => $self->localaddr) : (),
     defined $self->localport ? (LocalPort => $self->localport) : (),
@@ -203,6 +254,17 @@ sub send_email {
 
   while (length $msg_string) {
     my $next_hunk = substr $msg_string, 0, $hunk_size, '';
+
+    # For the need to downgrade, see
+    #   https://rt.cpan.org/Ticket/Display.html?id=104433
+    #
+    # The ||0 is there because when we've mocked Net::SMTP, there is no
+    # version.  We can't get the ->VERSION call to hit the mock, because we get
+    # the mock from ->new.  We don't want to create a new SMTP just to get the
+    # version, and we can't rely on $smtp being a Net::SMTP object.
+    # -- rjbs, 2015-08-10
+    utf8::downgrade($next_hunk) if (Net::SMTP->VERSION || 0) < 3.07;
+
     $smtp->datasend($next_hunk) or $FAULT->("error at during DATA");
   }
 
@@ -261,13 +323,13 @@ Email::Sender::Transport::SMTP - send email over SMTP
 
 =head1 VERSION
 
-version 1.300016
+version 1.300031
 
 =head1 DESCRIPTION
 
 This transport is used to send email over SMTP, either with or without secure
-sockets (SSL).  It is one of the most complex transports available, capable of
-partial success.
+sockets (SSL/TLS).  It is one of the most complex transports available, capable
+of partial success.
 
 For a potentially more efficient version of this transport, see
 L<Email::Sender::Transport::SMTP::Persistent>.
@@ -280,9 +342,15 @@ The following attributes may be passed to the constructor:
 
 =item C<host>: the name of the host to connect to; defaults to C<localhost>
 
-=item C<ssl>: if true, connect via SSL; defaults to false
+=item C<ssl>: if 'starttls', use STARTTLS; if 'ssl' (or 1), connect securely;
+otherwise, no security
 
-=item C<port>: port to connect to; defaults to 25 for non-SSL, 465 for SSL
+=item C<ssl_options>: passed to Net::SMTP constructor for 'ssl' connections or
+to starttls for 'starttls' connections; should contain extra options for
+IO::Socket::SSL
+
+=item C<port>: port to connect to; defaults to 25 for non-SSL, 465 for 'ssl',
+587 for 'starttls'
 
 =item C<timeout>: maximum time in secs to wait for server; default is 120
 
@@ -314,7 +382,7 @@ Ricardo Signes <rjbs@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Ricardo Signes.
+This software is copyright (c) 2017 by Ricardo Signes.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
